-- Different modes and units
--
--
-- Off:
--         No units
-- Standby:
--         No units
--
-- Safe:
--         Earth Sensor    = ES
--         Sun Sensor      = SS
--         Reaction Wheel  = RW
--
-- Nominal:
--         Star Tracker   = STR
--         GPS            = GPS (Coarse Navigation)
--         Reaction Wheel = RW
--         Thruster       = THR
--
-- Preparation:
--         All Nominal Mode units and
--         GPS (Fine Navigation)
--         Payload Instrument = PLI (Standby)
--
-- Science:
--         PLI (Science)


package Names is


   type Unit_ID_T is (
      ES,
      SS,
      RW,
      STR,
      GPS,
      THR,
      PLI );

   type Unit_Type_T is (
      Sensor,
      Actuator,
      Payload );

   Unit_Type : array (Unit_ID_T) of Unit_Type_T := (
      PLI      => Payload,
      RW | THR => Actuator,
      others   => Sensor );


   type Branch_ID_T is (A, B);

   type Branch_Status_T is (
      Free,
      Reserved,
      Locked );

   type Branch_State_T is (
      None,
      Off,
      On,
      GPS_Coarse_Navigation,
      GPS_Fine_Navigation,
      PLI_Standby,
      PLI_Science );
   
   Least_Non_Off_State : array (Unit_ID_T) of Branch_State_T := (
       GPS => GPS_Coarse_Navigation,
       PLI => PLI_Standby,
       others => On );

   subtype GPS_or_PLI_Branch_State_T is Branch_State_T range GPS_Coarse_Navigation..PLI_Science;

   type Error_ID_T is (
      None,
      Timeout,
      Loss_Of_Accuracy,
      Invalid_Data,
      Commanding_Failure,
      Attitude_Error );

   type Target_Mode_T is (
      None,
      Off,
      Standby,
      Safe,
      Nominal,
      Preparation,
      Science );

   subtype Mode_T is Target_Mode_T range Off..Science;

   type TC_ID_T is (
      GPS_Go_To_Coarse_Mode,
      GPS_Go_To_Fine_Mode,
      PLI_Go_To_Standby_Mode,
      PLI_Go_To_Science_Mode );

   Target_State_TC : array (GPS_or_PLI_Branch_State_T) of TC_ID_T := (
      GPS_Coarse_Navigation => GPS_Go_To_Coarse_Mode,
      GPS_Fine_Navigation => GPS_Go_To_Fine_Mode,
      PLI_Standby => PLI_Go_To_Standby_Mode,
      PLI_Science => PLI_Go_To_Science_Mode );

   type Event_ID_T is (
      GPS_In_Coarse_Mode,
      GPS_In_Fine_Mode,
      PLI_In_Standby_Mode,
      PLI_In_Science_Mode );

   Target_State_Event : array (GPS_or_PLI_Branch_State_T) of Event_ID_T := (
      GPS_Coarse_Navigation => GPS_In_Coarse_Mode,
      GPS_Fine_Navigation => GPS_In_Fine_Mode,
      PLI_Standby => PLI_In_Standby_Mode,
      PLI_Science => PLI_In_Science_Mode );

   type Branch_T is record
      State : Branch_State_T := Off;
      -- Invariant: No unexpected requests to change state
      --
      -- Note that state is Off or On, for all units except for
      -- GPS and PLI

      Status: Branch_Status_T := Free;
      -- Free, Reserved, Locked
      -- Invariant: To = Rerserved implies Br.Status = Free
      -- Invariant: To = Locked implies Br.Status = Reserved

      Target_State : Branch_State_T := None;

      Step  : Natural := 0;
      Timer : Natural := 0;

      Error : Error_ID_T := None;
      -- Invariant: For any BrId and AnotherBrId in Branch_ID_T,
      -- if BrId /= AnotherBrId and Unit.Br(BrId).Error /= None
      -- then Unit.Br(AnotherBrId).Error = None and Unit.CuBrId = BrId

      Error_Counter : Natural := 0;

      Timeout_Limit : Natural := 100;

   end record;

   type Branches_T is array (Branch_Id_T) of Branch_T;

   type Unit_T is record
      Br : Branches_T;
      CuBrId : Branch_Id_T := A;
      Reconf_Ongoing : Boolean := False;
      Orig_Status    : Branch_Status_T := Free;
      Target_State   : Branch_State_T := None;
      Step           : Natural := 0;
   end record;

   type Units_T is array (Unit_ID_T) of Unit_T;

end Names;
